/*
 *  cubesql.h
 *
 *	This file is the public interface for the cubeSQL Server SDK.
 *	You just need to include this header file in your projects.
 *
 *  (c) 2006-2011 SQLabs s.r.l. -- All Rights Reserved
 *  Author: Marco Bambini (MB)
 *
 */

#ifndef CUBESQLSDK_H
#define CUBESQLSDK_H

#ifdef __cplusplus
extern "C" {
#endif

// custom boolean values
#ifndef kTRUE
#define kTRUE				1
#endif

#ifndef kFALSE
#define kFALSE				0
#endif
	
// default values
#define	kDEFAULT_PORT		4430
#define kDEFAULT_TIMEOUT	12
#define kNOERR				0
#define kERR				-1

// client side errors
#define kMEMORY_ERROR		-2
#define kPARAMETER_ERROR	-3
#define kPROTOCOL_ERROR		-4
#define kZLIB_ERROR			-5

// encryption flags used in cubesql_connect
#define kAESNONE			0
#define kAES128				2
#define kAES192				3
#define kAES256				4
	
// flag used in cubesql_cursor_getfield
#define	kCUBESQL_COLNAME	0
#define kCUBESQL_CURROW		-1
#define	kCUBESQL_COLTABLE	-2
#define kCUBESQL_ROWID		-666
	
// flag used in cubesql_cursor_seek
#define kCUBESQL_SEEKNEXT	-2
#define kCUBESQL_SEEKFIRST	-3
#define kCUBESQL_SEEKLAST	-4
#define kCUBESQL_SEEKPREV	-5

#ifndef int64
#ifdef WIN32
typedef __int64 int64;
#else
typedef long long int int64;
#endif
#endif
	
// column types coming from the server
enum {
	TYPE_None		= 0,
	TYPE_Integer	= 1,
	TYPE_Float		= 2,
	TYPE_Text		= 3,
	TYPE_Blob		= 4,
	TYPE_Boolean	= 5,
	TYPE_Date		= 6,
	TYPE_Time		= 7,
	TYPE_Timestamp	= 8,
	TYPE_Currency	= 9
};

// column types to specify in the cubesql_bind command (coltype)
#define kBIND_INTEGER	1
#define kBIND_DOUBLE	2
#define kBIND_TEXT		3
#define kBIND_BLOB		4
#define kBIND_NULL		5
	
// define opaque datatypes
typedef struct csqldb csqldb;
typedef struct csqlc csqlc;
	
// function prototypes
int		cubesql_connect (csqldb **db, const char *host, int port, const char *username, const char *password, int timeout, int encryption);
void	cubesql_disconnect (csqldb *db, int gracefully);
int		cubesql_execute (csqldb *db, const char *sql);
csqlc	*cubesql_select (csqldb *db, const char *sql, int unused);
int		cubesql_commit (csqldb *db);
int		cubesql_rollback (csqldb *db);
int		cubesql_bind (csqldb *db, const char *sql, char **colvalue, int *colsize, int *coltype, int ncols);
int		cubesql_ping (csqldb *db);
int		cubesql_errcode (csqldb *db);
char	*cubesql_errmsg (csqldb *db);

int		cubesql_cursor_numrows (csqlc *c);
int		cubesql_cursor_numcolumns (csqlc *c);
int		cubesql_cursor_currentrow (csqlc *c);
int		cubesql_cursor_seek (csqlc *c, int index);
int		cubesql_cursor_iseof (csqlc *c);
int		cubesql_cursor_columntype (csqlc *c, int index);
char	*cubesql_cursor_field (csqlc *c, int row, int column, int *len);
int64	cubesql_cursor_rowid (csqlc *c, int row);
int64	cubesql_cursor_int64 (csqlc *c, int row, int column, int64 default_value);
int		cubesql_cursor_int (csqlc *c, int row, int column, int default_value);
double	cubesql_cursor_double (csqlc *c, int row, int column, double default_value);
char	*cubesql_cursor_cstring (csqlc *c, int row, int column);
char	*cubesql_cursor_cstring_static (csqlc *c, int row, int column, char *static_buffer, int bufferlen);	
void	cubesql_cursor_free (csqlc *c);

// private functions
int		cubesql_connect_token (csqldb **db, const char *host, int port, const char *username, const char *password, int timeout, int encryption, char *token);
void	cubesql_clear_errors (csqldb *db);
csqldb	*cubesql_cursor_db (csqlc *cursor);
csqlc	*cubesql_cursor_create (csqldb *db, int nrows, int ncolumns, int *types, char **names);
int		cubesql_cursor_addrow (csqlc *cursor, char **row, int *len);
int		cubesql_cursor_columntypebind (csqlc *c, int index);
void	cubesql_setuserptr (csqldb *db, void *userptr);
void	*cubesql_getuserptr (csqldb *db);
void	cubesql_settoken (csqldb *db, char *token);
char	*cubesql_gettoken (csqldb *db);
	
#ifdef __cplusplus
}  /* End of the 'extern "C"' block */
#endif

#endif
